;
;	TEXT-TO-PAL-PILOT TOKENIZER
;
;   Takes plain text PILOT programs/scripts and turns them into
; tokenized, loadable data files for use with the PAL PILOT
; interpreter, running under KIM-1 BASIC (KB9). By Dave Hassler, '23.
; MIT License.
;
;   CAVEAT: 1) The source text file *must* have only a CR as the end of
; line.  DOS CRLF and UNIX LF ending will break this.  Set line endings
; of the source text to CR-only.  2) The final line of the source
; text should be:  199 R:   This will indicate EOF.
;
;   Other bits of this seem to break randomly, but it generates the 
; data at $8000, where PAL PILOT looks for user programs.
;

* = $0200

SAL	= $E1
SAH	= $E2
GETCH	= $1E5A
CRLF	= $1E2F
STORLN	= $0400
TEMP1	= $E3
TEMP2	= $E4
STORY	= $E5


; INITIALIZE

	CLD		; why not?
	LDY #$00
	LDA #$00	; set up starting storage address
	STA SAL
	LDA #$80
	STA SAH

; PRINT TITLE

	JSR CRLF
 	LDX #MSG2-MSGBAS    ; Title msg
        JSR SNDMSG
	JSR CRLF


; GET A LINE OF PROGRAM TEXT

TAKEIN	LDX #$00
	STY STORY
LOOP	JSR GETCH	; get ASCII char from TTY 
	LDY STORY
	STA STORLN,X	; store it
	CMP #$0D	; is it a CR?
	BEQ NUMGET	; yes, line ended, calc line no.
	INX		; +1
	BNE LOOP	; can't have line over 72 char, so....


; GET THE LINE NUMBER

NUMGET	LDX #$00
LOOP1	LDA STORLN,X
	CMP #$20	; is it a SPC?
	BEQ COUNT	; yes, we're past the line #
	INX
	BNE LOOP1

COUNT	DEX		; go back one, to last digit
	LDA STORLN,X	; it's the '1s'
	SEC
	SBC #$30	; turn ASCII into a number
	STA TEMP1	; store it
	CPX #$00	; done?
	BEQ CHKEOF	; DONE
TENS	DEX		; x10 math code by Leo Nechaev, 2000
	LDA STORLN,X	; it's the '10s'
	SEC
	SBC #$30	; turn ASCII into a number
	ASL         	;multiply by 2
        STA TEMP2   	;temp store in TEMP2
        ASL         	;again multiply by 2 (*4)
        ASL         	;again multiply by 2 (*8)
        CLC
        ADC TEMP2   	;as result, A = x*8 + x*2
	ADC TEMP1	; add that to the 1s column
	STA TEMP1	; store it
	CPX #$00	; done?
	BEQ CHKEOF	; DONE
HUNDS	DEX
	LDA #$64	; #100 (can't have over 199 lines anyway)
	CLC
	ADC TEMP1	; add 100 to the line #
	STA TEMP1	; store it

CHKEOF	CMP #$C7	; is it #199 (proxy for true EOF $FF) ?
	BNE LINNUM	; no, store line # in user prog RAM
	LDA #$FF	; load EOF char
	JSR PUTBYT	; store byte in $8000+
	JSR CRLF
	LDX #MSG3-MSGBAS    ; display end addr msg
        JSR SNDMSG
	LDA SAH
	JSR $1E3B
	LDA SAL
	JSR $1E3B
	JSR CRLF
	JMP $1C4F	; ALL DONE! back to KIM-1 monitor START

LINNUM	CMP #$C8	; > #199?
	BCS INVAL	; yes, branch to invalid
	STA STORY	; no, store it and next module
	JSR PUTBYT
	JMP TOKGET

INVAL	JSR CRLF
	LDX #MSG4-MSGBAS    ; display invalid line # and quit
        JSR SNDMSG
	JSR CRLF
	JMP $1C4F	; START


; GET THE TOKEN AND CHECK FOR CONDITIONALS

TOKGET	LDX #$00
CHECK1	INX
	LDA STORLN,X
	CMP #$20	; is it SPC?
	BNE CHECK1	; no, get next char
	
	
TOKGO	LDY #$01	; init index to 1
	LDA STORLN+1,X	; get the statement letter
	CMP #$2A	; is it '*'?
	BEQ ISSTAR
	CMP #$45	; is it 'E'?
	BEQ ENDCHK
	STA TEMP1	; store statement letter in temp loc
GETTBL	LDA TABLE,Y	; get statement letter from table
	CMP TEMP1	; char = token from table?
	BEQ TOKSTR	; yes, go store it
	INY		; no, +1 Y and get next token
	CPY #$0A
	BEQ LEAVE
	BNE GETTBL	; unconditional branch


ENDCHK	STA TEMP1	; store A for possible BEQ to token check 
	LDA STORLN+2,X	; get next letter
	CMP #$4E	; is it 'N'?
	BNE GETTBL	; if it isn't, could be E: or EY:, go back
	LDA STORLN+3,X	; get third letter
	CMP #$44	; is it 'D'?
	BNE GETTBL	; if not, it's EN: and go back
	BEQ ISSTAR	; it is, so process as if for label

	
LEAVE	JSR CRLF
	LDX #MSG5-MSGBAS    ; display invalid token msg
        JSR SNDMSG
	LDA STORY
	JSR $1E3B	; output the line number
	JSR CRLF
	JMP $1C4F	; quit to KIM-1 monitor START

; PROCESS A LABEL LINE

ISSTAR	INX		; +1
	LDA STORLN,X	; get next char in line
	CMP #$0D	; is it CR?
	BEQ DUNSTA	; yes, prep to exit routine
	JSR PUTBYT	; put byte in user RAM
	BNE ISSTAR	; next char

DUNSTA	JSR PUTBYT	; finish the line
	JMP TAKEIN	; jump up to get next line of prog text

	
TOKSTR	STY TEMP2	; store token # in temp loc
	
	LDA STORLN+2,X	; get the character past token
	CMP #$59	; is it conditional Y?
	BNE CHECKN	; no, go check for N
	LDA #$0A	; #10
	CLC
	ADC TEMP2	; +10 for Y
	STA TEMP2	; store it
	BNE TOKOUT

CHECKN	LDA STORLN+2,X	; get the char past token
	CMP #$4E	; is it conditional N?
	BNE TOKOUT	; no, go output it
	LDA #$14	; #20
	CLC
	ADC TEMP2	; +20 for Y
	STA TEMP2	; store it

TOKOUT	LDA TEMP2
	JSR PUTBYT	; store it in user RAM


; GET THE OBJECT TEXT

	LDX #$00	; init index
LOOP2	LDA STORLN,X	; get char
	INX		; +1 index
	CMP #$3A	; is it ':'?
	BNE LOOP2	; no, get next char.
LOOP3	LDA STORLN,X	; get a char of object text
	STA TEMP1	; store in temp
	JSR PUTBYT	; and store in user RAM
	LDA TEMP1	; bring back the char
	INX		; bump index
	CMP #$0D	; is char CR?
	BNE LOOP3 	; no, get more
	JMP TAKEIN	; yes, go process the next line
	







; deposit a byte into $8000 user program area
			
PUTBYT	LDY #$00	; always 0
	STA (SAL),Y	; put byte in user RAM
	LDA #$01	; +1 SAL
	CLC
	ADC SAL
	STA SAL
	BCC OUT1	; if no carry, out
	LDA #$00	; yes, +1 SAH
	ADC SAH
	STA SAH
OUT1	RTS


; display message from table

SNDMSG	LDA MSGBAS,X
	BEQ RETRN2	
	JSR $1EA0
	INX
	BNE SNDMSG
RETRN2	RTS


TABLE
.BYTE	$00, $54, $4A, $45, $55, $4D, $43, $41, $49, $52

MSGBAS  = *

MSG2
.BYTE "PAL PILOT Tokenizer - Start TTY upload."
.BYTE $00

MSG3
.BYTE "Done. Start/End: $8000/$"
.BYTE $00

MSG4
.BYTE "***ERR: line # >199."
.BYTE $00

MSG5
.BYTE "Invalid statement letter in line $"
.BYTE $00
